/*
  File autogenerated by gengetopt version 2.22.5
  generated with the following command:
  gengetopt --input cmdline.ggo 

  The developers of gengetopt consider the fixed text that goes in all
  gengetopt output files to be in the public domain:
  we make no copyright claims on it.
*/

/* If we use autoconf.  */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef FIX_UNUSED
#define FIX_UNUSED(X) (void) (X) /* avoid warnings for unused params */
#endif

#include <getopt.h>

#include "cmdline.h"

const char *gengetopt_args_info_purpose = "Convert an IP to country code";

const char *gengetopt_args_info_usage = "Usage: ip_to_geo --ip <ipv4>|--daemon [--port <port>] [--bind-ip <ip>]";

const char *gengetopt_args_info_description = "Convert an IP to country code. Currently, onyl IPv4 supported";

const char *gengetopt_args_info_help[] = {
  "  -h, --help                    Print help and exit",
  "  -V, --version                 Print version and exit",
  "\nInteractive mode:",
  "  -i, --ip=ip                   IP to convert",
  "  -q, --quiet                   Quiet mode  (default=off)",
  "  -v, --verbose                 Verbose mode  (default=off)",
  "\nDaemon mode:",
  "  -D, --daemon                  Daemonize  (default=off)",
  "  -p, --port=INT                Port  (default=`53333')",
  "  -b, --bind-ip=STRING          IP to bind to",
  "  -B, --no-background           Don't go to background for daemon mode  \n                                  (default=off)",
  "\nAdvanced daemon options:",
  "  -s, --sockets-per-thread=INT  Number of sockets managed by a single thread  \n                                  (default=`10')",
  "  -t, --sockets-timeout=INT     Close connection after X seconds  (default=`5')",
  "  -r, --client-max-requests=INT Number of requests allowed for one client  \n                                  (default=`100')",
    0
};

typedef enum {ARG_NO
  , ARG_FLAG
  , ARG_STRING
  , ARG_INT
} cmdline_parser_arg_type;

static
void clear_given (struct gengetopt_args_info *args_info);
static
void clear_args (struct gengetopt_args_info *args_info);

static int
cmdline_parser_internal (int argc, char **argv, struct gengetopt_args_info *args_info,
                        struct cmdline_parser_params *params, const char *additional_error);


static char *
gengetopt_strdup (const char *s);

static
void clear_given (struct gengetopt_args_info *args_info)
{
  args_info->help_given = 0 ;
  args_info->version_given = 0 ;
  args_info->ip_given = 0 ;
  args_info->quiet_given = 0 ;
  args_info->verbose_given = 0 ;
  args_info->daemon_given = 0 ;
  args_info->port_given = 0 ;
  args_info->bind_ip_given = 0 ;
  args_info->no_background_given = 0 ;
  args_info->sockets_per_thread_given = 0 ;
  args_info->sockets_timeout_given = 0 ;
  args_info->client_max_requests_given = 0 ;
}

static
void clear_args (struct gengetopt_args_info *args_info)
{
  FIX_UNUSED (args_info);
  args_info->ip_arg = NULL;
  args_info->ip_orig = NULL;
  args_info->quiet_flag = 0;
  args_info->verbose_flag = 0;
  args_info->daemon_flag = 0;
  args_info->port_arg = 53333;
  args_info->port_orig = NULL;
  args_info->bind_ip_arg = NULL;
  args_info->bind_ip_orig = NULL;
  args_info->no_background_flag = 0;
  args_info->sockets_per_thread_arg = 10;
  args_info->sockets_per_thread_orig = NULL;
  args_info->sockets_timeout_arg = 5;
  args_info->sockets_timeout_orig = NULL;
  args_info->client_max_requests_arg = 100;
  args_info->client_max_requests_orig = NULL;
  
}

static
void init_args_info(struct gengetopt_args_info *args_info)
{


  args_info->help_help = gengetopt_args_info_help[0] ;
  args_info->version_help = gengetopt_args_info_help[1] ;
  args_info->ip_help = gengetopt_args_info_help[3] ;
  args_info->quiet_help = gengetopt_args_info_help[4] ;
  args_info->verbose_help = gengetopt_args_info_help[5] ;
  args_info->daemon_help = gengetopt_args_info_help[7] ;
  args_info->port_help = gengetopt_args_info_help[8] ;
  args_info->bind_ip_help = gengetopt_args_info_help[9] ;
  args_info->no_background_help = gengetopt_args_info_help[10] ;
  args_info->sockets_per_thread_help = gengetopt_args_info_help[12] ;
  args_info->sockets_timeout_help = gengetopt_args_info_help[13] ;
  args_info->client_max_requests_help = gengetopt_args_info_help[14] ;
  
}

void
cmdline_parser_print_version (void)
{
  printf ("%s %s\n",
     (strlen(CMDLINE_PARSER_PACKAGE_NAME) ? CMDLINE_PARSER_PACKAGE_NAME : CMDLINE_PARSER_PACKAGE),
     CMDLINE_PARSER_VERSION);
}

static void print_help_common(void) {
  cmdline_parser_print_version ();

  if (strlen(gengetopt_args_info_purpose) > 0)
    printf("\n%s\n", gengetopt_args_info_purpose);

  if (strlen(gengetopt_args_info_usage) > 0)
    printf("\n%s\n", gengetopt_args_info_usage);

  printf("\n");

  if (strlen(gengetopt_args_info_description) > 0)
    printf("%s\n\n", gengetopt_args_info_description);
}

void
cmdline_parser_print_help (void)
{
  int i = 0;
  print_help_common();
  while (gengetopt_args_info_help[i])
    printf("%s\n", gengetopt_args_info_help[i++]);
}

void
cmdline_parser_init (struct gengetopt_args_info *args_info)
{
  clear_given (args_info);
  clear_args (args_info);
  init_args_info (args_info);
}

void
cmdline_parser_params_init(struct cmdline_parser_params *params)
{
  if (params)
    { 
      params->override = 0;
      params->initialize = 1;
      params->check_required = 1;
      params->check_ambiguity = 0;
      params->print_errors = 1;
    }
}

struct cmdline_parser_params *
cmdline_parser_params_create(void)
{
  struct cmdline_parser_params *params = 
    (struct cmdline_parser_params *)malloc(sizeof(struct cmdline_parser_params));
  cmdline_parser_params_init(params);  
  return params;
}

static void
free_string_field (char **s)
{
  if (*s)
    {
      free (*s);
      *s = 0;
    }
}


static void
cmdline_parser_release (struct gengetopt_args_info *args_info)
{

  free_string_field (&(args_info->ip_arg));
  free_string_field (&(args_info->ip_orig));
  free_string_field (&(args_info->port_orig));
  free_string_field (&(args_info->bind_ip_arg));
  free_string_field (&(args_info->bind_ip_orig));
  free_string_field (&(args_info->sockets_per_thread_orig));
  free_string_field (&(args_info->sockets_timeout_orig));
  free_string_field (&(args_info->client_max_requests_orig));
  
  

  clear_given (args_info);
}


static void
write_into_file(FILE *outfile, const char *opt, const char *arg, const char *values[])
{
  FIX_UNUSED (values);
  if (arg) {
    fprintf(outfile, "%s=\"%s\"\n", opt, arg);
  } else {
    fprintf(outfile, "%s\n", opt);
  }
}


int
cmdline_parser_dump(FILE *outfile, struct gengetopt_args_info *args_info)
{
  int i = 0;

  if (!outfile)
    {
      fprintf (stderr, "%s: cannot dump options to stream\n", CMDLINE_PARSER_PACKAGE);
      return EXIT_FAILURE;
    }

  if (args_info->help_given)
    write_into_file(outfile, "help", 0, 0 );
  if (args_info->version_given)
    write_into_file(outfile, "version", 0, 0 );
  if (args_info->ip_given)
    write_into_file(outfile, "ip", args_info->ip_orig, 0);
  if (args_info->quiet_given)
    write_into_file(outfile, "quiet", 0, 0 );
  if (args_info->verbose_given)
    write_into_file(outfile, "verbose", 0, 0 );
  if (args_info->daemon_given)
    write_into_file(outfile, "daemon", 0, 0 );
  if (args_info->port_given)
    write_into_file(outfile, "port", args_info->port_orig, 0);
  if (args_info->bind_ip_given)
    write_into_file(outfile, "bind-ip", args_info->bind_ip_orig, 0);
  if (args_info->no_background_given)
    write_into_file(outfile, "no-background", 0, 0 );
  if (args_info->sockets_per_thread_given)
    write_into_file(outfile, "sockets-per-thread", args_info->sockets_per_thread_orig, 0);
  if (args_info->sockets_timeout_given)
    write_into_file(outfile, "sockets-timeout", args_info->sockets_timeout_orig, 0);
  if (args_info->client_max_requests_given)
    write_into_file(outfile, "client-max-requests", args_info->client_max_requests_orig, 0);
  

  i = EXIT_SUCCESS;
  return i;
}

int
cmdline_parser_file_save(const char *filename, struct gengetopt_args_info *args_info)
{
  FILE *outfile;
  int i = 0;

  outfile = fopen(filename, "w");

  if (!outfile)
    {
      fprintf (stderr, "%s: cannot open file for writing: %s\n", CMDLINE_PARSER_PACKAGE, filename);
      return EXIT_FAILURE;
    }

  i = cmdline_parser_dump(outfile, args_info);
  fclose (outfile);

  return i;
}

void
cmdline_parser_free (struct gengetopt_args_info *args_info)
{
  cmdline_parser_release (args_info);
}

/** @brief replacement of strdup, which is not standard */
char *
gengetopt_strdup (const char *s)
{
  char *result = 0;
  if (!s)
    return result;

  result = (char*)malloc(strlen(s) + 1);
  if (result == (char*)0)
    return (char*)0;
  strcpy(result, s);
  return result;
}

int
cmdline_parser (int argc, char **argv, struct gengetopt_args_info *args_info)
{
  return cmdline_parser2 (argc, argv, args_info, 0, 1, 1);
}

int
cmdline_parser_ext (int argc, char **argv, struct gengetopt_args_info *args_info,
                   struct cmdline_parser_params *params)
{
  int result;
  result = cmdline_parser_internal (argc, argv, args_info, params, 0);

  if (result == EXIT_FAILURE)
    {
      cmdline_parser_free (args_info);
      exit (EXIT_FAILURE);
    }
  
  return result;
}

int
cmdline_parser2 (int argc, char **argv, struct gengetopt_args_info *args_info, int override, int initialize, int check_required)
{
  int result;
  struct cmdline_parser_params params;
  
  params.override = override;
  params.initialize = initialize;
  params.check_required = check_required;
  params.check_ambiguity = 0;
  params.print_errors = 1;

  result = cmdline_parser_internal (argc, argv, args_info, &params, 0);

  if (result == EXIT_FAILURE)
    {
      cmdline_parser_free (args_info);
      exit (EXIT_FAILURE);
    }
  
  return result;
}

int
cmdline_parser_required (struct gengetopt_args_info *args_info, const char *prog_name)
{
  FIX_UNUSED (args_info);
  FIX_UNUSED (prog_name);
  return EXIT_SUCCESS;
}


static char *package_name = 0;

/**
 * @brief updates an option
 * @param field the generic pointer to the field to update
 * @param orig_field the pointer to the orig field
 * @param field_given the pointer to the number of occurrence of this option
 * @param prev_given the pointer to the number of occurrence already seen
 * @param value the argument for this option (if null no arg was specified)
 * @param possible_values the possible values for this option (if specified)
 * @param default_value the default value (in case the option only accepts fixed values)
 * @param arg_type the type of this option
 * @param check_ambiguity @see cmdline_parser_params.check_ambiguity
 * @param override @see cmdline_parser_params.override
 * @param no_free whether to free a possible previous value
 * @param multiple_option whether this is a multiple option
 * @param long_opt the corresponding long option
 * @param short_opt the corresponding short option (or '-' if none)
 * @param additional_error possible further error specification
 */
static
int update_arg(void *field, char **orig_field,
               unsigned int *field_given, unsigned int *prev_given, 
               char *value, const char *possible_values[],
               const char *default_value,
               cmdline_parser_arg_type arg_type,
               int check_ambiguity, int override,
               int no_free, int multiple_option,
               const char *long_opt, char short_opt,
               const char *additional_error)
{
  char *stop_char = 0;
  const char *val = value;
  int found;
  char **string_field;
  FIX_UNUSED (field);

  stop_char = 0;
  found = 0;

  if (!multiple_option && prev_given && (*prev_given || (check_ambiguity && *field_given)))
    {
      if (short_opt != '-')
        fprintf (stderr, "%s: `--%s' (`-%c') option given more than once%s\n", 
               package_name, long_opt, short_opt,
               (additional_error ? additional_error : ""));
      else
        fprintf (stderr, "%s: `--%s' option given more than once%s\n", 
               package_name, long_opt,
               (additional_error ? additional_error : ""));
      return 1; /* failure */
    }

  FIX_UNUSED (default_value);
    
  if (field_given && *field_given && ! override)
    return 0;
  if (prev_given)
    (*prev_given)++;
  if (field_given)
    (*field_given)++;
  if (possible_values)
    val = possible_values[found];

  switch(arg_type) {
  case ARG_FLAG:
    *((int *)field) = !*((int *)field);
    break;
  case ARG_INT:
    if (val) *((int *)field) = strtol (val, &stop_char, 0);
    break;
  case ARG_STRING:
    if (val) {
      string_field = (char **)field;
      if (!no_free && *string_field)
        free (*string_field); /* free previous string */
      *string_field = gengetopt_strdup (val);
    }
    break;
  default:
    break;
  };

  /* check numeric conversion */
  switch(arg_type) {
  case ARG_INT:
    if (val && !(stop_char && *stop_char == '\0')) {
      fprintf(stderr, "%s: invalid numeric value: %s\n", package_name, val);
      return 1; /* failure */
    }
    break;
  default:
    ;
  };

  /* store the original value */
  switch(arg_type) {
  case ARG_NO:
  case ARG_FLAG:
    break;
  default:
    if (value && orig_field) {
      if (no_free) {
        *orig_field = value;
      } else {
        if (*orig_field)
          free (*orig_field); /* free previous string */
        *orig_field = gengetopt_strdup (value);
      }
    }
  };

  return 0; /* OK */
}


int
cmdline_parser_internal (
  int argc, char **argv, struct gengetopt_args_info *args_info,
                        struct cmdline_parser_params *params, const char *additional_error)
{
  int c;	/* Character of the parsed option.  */

  int error = 0;
  struct gengetopt_args_info local_args_info;
  
  int override;
  int initialize;
  int check_required;
  int check_ambiguity;
  
  package_name = argv[0];
  
  override = params->override;
  initialize = params->initialize;
  check_required = params->check_required;
  check_ambiguity = params->check_ambiguity;

  if (initialize)
    cmdline_parser_init (args_info);

  cmdline_parser_init (&local_args_info);

  optarg = 0;
  optind = 0;
  opterr = params->print_errors;
  optopt = '?';

  while (1)
    {
      int option_index = 0;

      static struct option long_options[] = {
        { "help",	0, NULL, 'h' },
        { "version",	0, NULL, 'V' },
        { "ip",	1, NULL, 'i' },
        { "quiet",	0, NULL, 'q' },
        { "verbose",	0, NULL, 'v' },
        { "daemon",	0, NULL, 'D' },
        { "port",	1, NULL, 'p' },
        { "bind-ip",	1, NULL, 'b' },
        { "no-background",	0, NULL, 'B' },
        { "sockets-per-thread",	1, NULL, 's' },
        { "sockets-timeout",	1, NULL, 't' },
        { "client-max-requests",	1, NULL, 'r' },
        { 0,  0, 0, 0 }
      };

      c = getopt_long (argc, argv, "hVi:qvDp:b:Bs:t:r:", long_options, &option_index);

      if (c == -1) break;	/* Exit from `while (1)' loop.  */

      switch (c)
        {
        case 'h':	/* Print help and exit.  */
          cmdline_parser_print_help ();
          cmdline_parser_free (&local_args_info);
          exit (EXIT_SUCCESS);

        case 'V':	/* Print version and exit.  */
          cmdline_parser_print_version ();
          cmdline_parser_free (&local_args_info);
          exit (EXIT_SUCCESS);

        case 'i':	/* IP to convert.  */
        
        
          if (update_arg( (void *)&(args_info->ip_arg), 
               &(args_info->ip_orig), &(args_info->ip_given),
              &(local_args_info.ip_given), optarg, 0, 0, ARG_STRING,
              check_ambiguity, override, 0, 0,
              "ip", 'i',
              additional_error))
            goto failure;
        
          break;
        case 'q':	/* Quiet mode.  */
        
        
          if (update_arg((void *)&(args_info->quiet_flag), 0, &(args_info->quiet_given),
              &(local_args_info.quiet_given), optarg, 0, 0, ARG_FLAG,
              check_ambiguity, override, 1, 0, "quiet", 'q',
              additional_error))
            goto failure;
        
          break;
        case 'v':	/* Verbose mode.  */
        
        
          if (update_arg((void *)&(args_info->verbose_flag), 0, &(args_info->verbose_given),
              &(local_args_info.verbose_given), optarg, 0, 0, ARG_FLAG,
              check_ambiguity, override, 1, 0, "verbose", 'v',
              additional_error))
            goto failure;
        
          break;
        case 'D':	/* Daemonize.  */
        
        
          if (update_arg((void *)&(args_info->daemon_flag), 0, &(args_info->daemon_given),
              &(local_args_info.daemon_given), optarg, 0, 0, ARG_FLAG,
              check_ambiguity, override, 1, 0, "daemon", 'D',
              additional_error))
            goto failure;
        
          break;
        case 'p':	/* Port.  */
        
        
          if (update_arg( (void *)&(args_info->port_arg), 
               &(args_info->port_orig), &(args_info->port_given),
              &(local_args_info.port_given), optarg, 0, "53333", ARG_INT,
              check_ambiguity, override, 0, 0,
              "port", 'p',
              additional_error))
            goto failure;
        
          break;
        case 'b':	/* IP to bind to.  */
        
        
          if (update_arg( (void *)&(args_info->bind_ip_arg), 
               &(args_info->bind_ip_orig), &(args_info->bind_ip_given),
              &(local_args_info.bind_ip_given), optarg, 0, 0, ARG_STRING,
              check_ambiguity, override, 0, 0,
              "bind-ip", 'b',
              additional_error))
            goto failure;
        
          break;
        case 'B':	/* Don't go to background for daemon mode.  */


          if (update_arg((void *)&(args_info->no_background_flag), 0, &(args_info->no_background_given),
              &(local_args_info.no_background_given), optarg, 0, 0, ARG_FLAG,
              check_ambiguity, override, 1, 0, "no-background", 'B',
              additional_error))
            goto failure;

          break;
        case 's':	/* Number of sockets managed by a single thread.  */
        
        
          if (update_arg( (void *)&(args_info->sockets_per_thread_arg), 
               &(args_info->sockets_per_thread_orig), &(args_info->sockets_per_thread_given),
              &(local_args_info.sockets_per_thread_given), optarg, 0, "10", ARG_INT,
              check_ambiguity, override, 0, 0,
              "sockets-per-thread", 's',
              additional_error))
            goto failure;
        
          break;
        case 't':	/* Close connection after X seconds.  */
        
        
          if (update_arg( (void *)&(args_info->sockets_timeout_arg), 
               &(args_info->sockets_timeout_orig), &(args_info->sockets_timeout_given),
              &(local_args_info.sockets_timeout_given), optarg, 0, "5", ARG_INT,
              check_ambiguity, override, 0, 0,
              "sockets-timeout", 't',
              additional_error))
            goto failure;
        
          break;
        case 'r':	/* Number of requests allowed for one client.  */
        
        
          if (update_arg( (void *)&(args_info->client_max_requests_arg), 
               &(args_info->client_max_requests_orig), &(args_info->client_max_requests_given),
              &(local_args_info.client_max_requests_given), optarg, 0, "100", ARG_INT,
              check_ambiguity, override, 0, 0,
              "client-max-requests", 'r',
              additional_error))
            goto failure;
        
          break;

        case 0:	/* Long option with no short option */
        case '?':	/* Invalid option.  */
          /* `getopt_long' already printed an error message.  */
          goto failure;

        default:	/* bug: option not considered.  */
          fprintf (stderr, "%s: option unknown: %c%s\n", CMDLINE_PARSER_PACKAGE, c, (additional_error ? additional_error : ""));
          abort ();
        } /* switch */
    } /* while */




  cmdline_parser_release (&local_args_info);

  if ( error )
    return (EXIT_FAILURE);

  return 0;

failure:
  
  cmdline_parser_release (&local_args_info);
  return (EXIT_FAILURE);
}
